import { BuiltinPlugin, __toESM, buildImportAnalysisPlugin, composeJsPlugins, createBundler, dynamicImportVarsPlugin, handleOutputErrors, importGlobPlugin, jsonPlugin, loadFallbackPlugin, manifestPlugin, modulePreloadPolyfillPlugin, normalizedStringOrRegex, require_binding, viteResolvePlugin, wasmFallbackPlugin, wasmHelperPlugin } from "../shared/src-Bb3zZkPW.mjs";
import { pathToFileURL } from "node:url";

//#region src/plugin/parallel-plugin.ts
function defineParallelPlugin(pluginPath) {
	return (options) => {
		return { _parallel: {
			fileUrl: pathToFileURL(pluginPath).href,
			options
		} };
	};
}

//#endregion
//#region src/api/experimental.ts
const experimental_scan = async (input) => {
	const { bundler, stopWorkers } = await createBundler(input, {});
	const output = await bundler.scan();
	handleOutputErrors(output);
	await stopWorkers?.();
};

//#endregion
//#region src/builtin-plugin/transform-plugin.ts
function normalizeEcmaTransformPluginConfig(config) {
	if (!config) return undefined;
	let normalizedConfig = {
		...config,
		exclude: normalizedStringOrRegex(config.exclude),
		include: normalizedStringOrRegex(config.include)
	};
	return normalizedConfig;
}
function transformPlugin(config) {
	return new BuiltinPlugin("builtin:transform", normalizeEcmaTransformPluginConfig(config));
}

//#endregion
//#region src/builtin-plugin/replace-plugin.ts
function replacePlugin(values = {}, options = {}) {
	return new BuiltinPlugin("builtin:replace", {
		...options,
		values
	});
}

//#endregion
//#region src/builtin-plugin/alias-plugin.ts
function aliasPlugin(config) {
	return new BuiltinPlugin("builtin:alias", config);
}

//#endregion
//#region src/experimental-index.ts
var import_binding = __toESM(require_binding());

//#endregion
var transform = import_binding.transform;
export { aliasPlugin, buildImportAnalysisPlugin, composeJsPlugins as composePlugins, defineParallelPlugin, dynamicImportVarsPlugin, importGlobPlugin, jsonPlugin, loadFallbackPlugin, manifestPlugin, modulePreloadPolyfillPlugin, replacePlugin, experimental_scan as scan, transform, transformPlugin, viteResolvePlugin, wasmFallbackPlugin, wasmHelperPlugin };